package net.jakewilliamson.sentry;

import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.view.Gravity;
import android.view.View;
import android.widget.Toast;

import java.util.Calendar;

import mehdi.sakout.aboutpage.AboutPage;
import mehdi.sakout.aboutpage.Element;

import static net.jakewilliamson.sentry.Util.getSelfVersionCode;
import static net.jakewilliamson.sentry.Util.getSelfVersionName;


public class About extends AppCompatActivity
{
    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        String versionName = getSelfVersionName(this);
        Integer versionNumber = getSelfVersionCode(this);

        View aboutPage = new AboutPage(this)
                .isRTL(false)
                .setDescription(" ")
                .setImage(R.drawable.about_logo)
                .addGroup("Acknowledgements")
                .addItem(getAckElement("Hexene", "https://github.com/hexene/LocalVPN"))
                .addItem(getAckElement("Material.io", "https://material.io"))
                .addItem(getAckElement("Medyo", "https://github.com/medyo/android-about-page"))
                .addGroup("Version")
                .addItem(new Element().setTitle("Name: " + versionName))
                .addItem(new Element().setTitle("Code: " + versionNumber))
                .addItem(getCopyRightsElement())
                .create();

        setContentView(aboutPage);
    }

    private Element getAckElement(String title, final String url)
    {
        Element element = new Element();
        element.setTitle(title);
        element.setOnClickListener(new View.OnClickListener()
        {
            @Override
            public void onClick(View v)
            {
                Intent browserIntent = new Intent(Intent.ACTION_VIEW, Uri.parse(url));
                startActivity(browserIntent);
            }
        });
        return element;
    }

    private Element getCopyRightsElement()
    {
        Element copyRightsElement = new Element();
        final String copyrights = String.format(getString(R.string.copy_right), Calendar.getInstance().get(Calendar.YEAR));
        copyRightsElement.setTitle(copyrights);
        copyRightsElement.setIconDrawable(R.drawable.ic_copyright_black_24dp);
        copyRightsElement.setIconTint(mehdi.sakout.aboutpage.R.color.about_item_icon_color);
        copyRightsElement.setIconNightTint(android.R.color.white);
        copyRightsElement.setGravity(Gravity.CENTER);
        copyRightsElement.setOnClickListener(new View.OnClickListener()
        {
            @Override
            public void onClick(View v)
            {
                Toast.makeText(About.this, copyrights, Toast.LENGTH_SHORT).show();
            }
        });
        return copyRightsElement;
    }
}