package net.jakewilliamson.sentry;

import android.annotation.SuppressLint;
import android.app.AlertDialog;
import android.app.DatePickerDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.support.design.widget.TextInputEditText;
import android.support.design.widget.TextInputLayout;
import android.support.v7.app.AppCompatActivity;
import android.text.Html;
import android.text.InputType;
import android.text.Spanned;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CheckBox;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.RadioGroup;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Objects;

import static android.text.InputType.TYPE_DATETIME_VARIATION_DATE;
import static android.text.InputType.TYPE_TEXT_VARIATION_EMAIL_ADDRESS;

public class EditData extends AppCompatActivity implements View.OnClickListener
{
    private DatePickerDialog dobDialog;
    private SimpleDateFormat dateFormatter;
    private final List<String> hintList = new ArrayList<>();
    private final List<String> valueList = new ArrayList<>();
    private final List<Integer> typeList = new ArrayList<>();
    private final ArrayList<String> reserved = new ArrayList<>();
    private final ArrayList<String> toDelete = new ArrayList<>();

    private EditText dobText;

    @SuppressWarnings("deprecation")
    private static Spanned fromHtml(String html)
    {
        return Html.fromHtml(html, Html.FROM_HTML_MODE_LEGACY);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_edit_data);

        reserved.add("Name");
        reserved.add("Email");
        reserved.add("Date of Birth");
        reserved.add("Phone");

        loadData();

        dateFormatter = new SimpleDateFormat("dd-MM-yyyy", Locale.US);

        findViewByID();
        setDateField();
    }

    private void findViewByID()
    {
        dobText = findViewById(R.id.dob);
        dobText.setInputType(InputType.TYPE_NULL);
        dobText.requestFocus();
    }

    private void setDateField()
    {
        dobText.setOnClickListener(this);

        Calendar calendar = Calendar.getInstance();
        dobDialog = new DatePickerDialog(this, new DatePickerDialog.OnDateSetListener()
        {
            public void onDateSet(DatePicker view, int year, int month, int day)
            {
                Calendar newDate = Calendar.getInstance();

                newDate.set(year, month, day);
                dobText.setText(dateFormatter.format(newDate.getTime()));
            }
        }, calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), calendar.get(Calendar.DAY_OF_MONTH));
    }

    @Override
    public void onClick(View v)
    {
        dobDialog.show();
    }

    public void showPromptFromButton(View v)
    {
        showPrompt(v, "");
    }

    private void showPrompt(final View v, String defValue)
    {
        LayoutInflater inflaterPrompt = LayoutInflater.from(this);
        @SuppressLint("InflateParams") final View promptsView = inflaterPrompt.inflate(R.layout.add_row_prompt, null);
        AlertDialog.Builder builder = new AlertDialog.Builder(this);

        builder.setView(promptsView);
        final EditText input = promptsView.findViewById(R.id.userInput1);

        if (!defValue.equals(""))
        {
            input.setText(defValue);
        }

        builder
                .setCancelable(false)
                .setPositiveButton("OK", new DialogInterface.OnClickListener()
                {
                    @Override
                    public void onClick(DialogInterface dialog, int which)
                    {
                        String hint = input.getText().toString();
                        boolean canAdd = true;
                        for (Object name : reserved)
                        {
                            if (hint.equals(name))
                            {
                                canAdd = false;
                                break;
                            }
                        }
                        if (canAdd)
                        {
                            Spinner spinner = promptsView.findViewById(R.id.spinner2);
                            String type = String.valueOf(spinner.getSelectedItem());
                            int fieldType;

                            switch (type)
                            {
                                case "Phone":
                                    fieldType = InputType.TYPE_CLASS_PHONE;
                                    break;

                                case "Email":
                                    fieldType = InputType.TYPE_CLASS_TEXT | TYPE_TEXT_VARIATION_EMAIL_ADDRESS;
                                    break;

                                case "Date":
                                    fieldType = InputType.TYPE_CLASS_DATETIME | TYPE_DATETIME_VARIATION_DATE;
                                    break;

                                case "Password":
                                    fieldType = InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_PASSWORD;
                                    break;

                                case "Text":
                                default:
                                    fieldType = InputType.TYPE_TEXT_FLAG_CAP_WORDS;
                                    break;
                            }

                            addRow(hint, "", fieldType);
                        } else
                        {
                            showError(v, hint);
                        }
                    }
                })
                .setNegativeButton("Cancel", new DialogInterface.OnClickListener()
                {
                    @Override
                    public void onClick(DialogInterface dialog, int which)
                    {
                        dialog.cancel();
                    }
                });
        AlertDialog alertDialog = builder.create();
        alertDialog.show();
    }

    private void showError(final View v, final String hint)
    {
        LayoutInflater inflaterPrompt = LayoutInflater.from(this);
        @SuppressLint("InflateParams") View promptsView = inflaterPrompt.inflate(R.layout.cancel_add_row, null);
        AlertDialog.Builder builder = new AlertDialog.Builder(this);

        builder.setView(promptsView);
        TextView text = promptsView.findViewById(R.id.textView1);

        String sourceString = "<i>" + hint + "</i> is already used, please try something else.";

        text.setText(fromHtml(sourceString));

        builder
                .setCancelable(false)
                .setPositiveButton("OK", new DialogInterface.OnClickListener()
                {
                    @Override
                    public void onClick(DialogInterface dialog, int which)
                    {
                        showPrompt(v, hint);
                    }
                });
        AlertDialog alertDialog = builder.create();
        alertDialog.show();
    }

    private void addRow(String hint, String value, Integer fieldType)
    {
        LayoutInflater inflater = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        final View rowView = inflater.inflate(R.layout.field, null);
        TextInputEditText newEdit = rowView.findViewById(R.id.newText);
        TextInputLayout newLayout = rowView.findViewById(R.id.newLayout);
        newLayout.setHint(hint);
        newEdit.setHint(hint);
        if (!value.equals(""))
        {
            newEdit.setText(value);
        }

        newEdit.setInputType(fieldType);

        LinearLayout parent = findViewById(R.id.mainLayout);
        parent.addView(rowView, parent.getChildCount() - 1);
        reserved.add(hint);
    }

    public void onSave(View v)
    {
        saveData();
    }

    private void getValueRec(ViewGroup root)
    {
        String value = "";
        String hint = "";

        for (int i = 0, j = root.getChildCount(); i < j; i++)
        {
            View view = root.getChildAt(i);
            if (view instanceof ViewGroup)
            {
                getValueRec((ViewGroup) view);
                continue;
            }
            if (view instanceof TextInputEditText)
            {
                value = ((TextInputEditText) view).getText().toString();
                TextInputLayout parent = (TextInputLayout) view.getParent().getParent();
                hint = Objects.requireNonNull(parent.getHint()).toString();
            }
            if (!value.equals(""))
            {
                try
                {
                    valueList.add(URLEncoder.encode(value, "UTF-8"));
                    hintList.add(hint);
                    typeList.add(((TextInputEditText) view).getInputType());
                } catch (UnsupportedEncodingException e)
                {
                    //Ignore
                }
            }
        }
    }

    private void saveData()
    {
        final String PREF_NAME = this.getString(R.string.app_name) + "_pref";
        final String DATA_TYPE = this.getString(R.string.app_name) + "data_type_pref";
        SharedPreferences pref = this.getSharedPreferences(PREF_NAME, Context.MODE_PRIVATE);
        SharedPreferences prefType = this.getSharedPreferences(DATA_TYPE, Context.MODE_PRIVATE);

        SharedPreferences.Editor editor = pref.edit();
        SharedPreferences.Editor editorTypes = prefType.edit();

        LinearLayout ll = this.findViewById(R.id.mainLayout);
        getValueRec(ll);

        for (int i = 0; i < valueList.size(); i++)
        {
            editor.putString(hintList.get(i), valueList.get(i));
            editorTypes.putInt(hintList.get(i), typeList.get(i));
        }

        editor.apply();
        editorTypes.apply();

        Toast.makeText(getApplicationContext(), "Data Saved",
                Toast.LENGTH_SHORT).show();

        finish();
    }

    public void prepDelete(View v)
    {
        LayoutInflater inflaterPrompt = LayoutInflater.from(this);
        final LinearLayout promptsView = (LinearLayout) inflaterPrompt.inflate(R.layout.delete_rows, null);
        AlertDialog.Builder builder = new AlertDialog.Builder(this);

        final boolean doDelete;

        if (4 < reserved.size())
        {
            doDelete = true;
            for (int i = 4; i < reserved.size(); i++)
            {
                CheckBox cb = new CheckBox(getApplicationContext());
                cb.setText(reserved.get(i));
                promptsView.addView(cb);
            }
        } else
        {
            doDelete = false;
            TextView text = promptsView.findViewById(R.id.textView1);
            text.setText(R.string.noDelete);
        }

        builder.setView(promptsView);

        builder
                .setCancelable(false)
                .setPositiveButton("OK", new DialogInterface.OnClickListener()
                {
                    @Override
                    public void onClick(DialogInterface dialog, int which)
                    {
                        if (doDelete)
                        {
                            getFieldsForDelete(promptsView);
                            LinearLayout parent = findViewById(R.id.mainLayout);
                            deleteFields(parent);
                        } else
                        {
                            dialog.cancel();
                        }
                    }
                })
                .setNegativeButton("Cancel", new DialogInterface.OnClickListener()
                {
                    @Override
                    public void onClick(DialogInterface dialog, int which)
                    {
                        dialog.cancel();
                    }
                });
        AlertDialog alertDialog = builder.create();
        alertDialog.show();
    }

    private void getFieldsForDelete(ViewGroup root)
    {
        for (int i = 0, j = root.getChildCount(); i < j; i++)
        {
            View view = root.getChildAt(i);
            if (view instanceof ViewGroup)
            {
                getFieldsForDelete((ViewGroup) view);
                continue;
            }
            if (view instanceof CheckBox)
            {
                if (((CheckBox) view).isChecked())
                {
                    toDelete.add(((CheckBox) view).getText().toString());
                }
            }
        }
    }

    private void deleteFields(ViewGroup root)
    {
        for (int i = 0, j = root.getChildCount(); i < j; i++)
        {
            View view = root.getChildAt(i);
            if (view instanceof ViewGroup)
            {
                deleteFields((ViewGroup) view);
                continue;
            }
            if (view instanceof TextInputEditText)
            {
                TextInputLayout parent = (TextInputLayout) view.getParent().getParent();
                String hint = parent.getHint().toString();
                if (toDelete.contains(hint))
                {
                    final String PREF_NAME = this.getString(R.string.app_name) + "_pref";
                    SharedPreferences pref = this.getSharedPreferences(PREF_NAME, Context.MODE_PRIVATE);

                    SharedPreferences.Editor editor = pref.edit();

                    parent.removeAllViews();

                    editor.remove(hint);
                    editor.apply();
                }
            }
        }
    }

    private void loadData()
    {
        final String PREF_NAME = this.getString(R.string.app_name) + "_pref";
        final String DATA_TYPE = this.getString(R.string.app_name) + "data_type_pref";
        SharedPreferences pref = this.getSharedPreferences(PREF_NAME, Context.MODE_PRIVATE);
        SharedPreferences prefType = this.getSharedPreferences(DATA_TYPE, Context.MODE_PRIVATE);

        HashMap kvs = (HashMap) pref.getAll();
        HashMap types = (HashMap) prefType.getAll();

        int savedItemsCount = kvs.size() - 4;

        if (0 < savedItemsCount)
        {
            for (Object key : kvs.keySet())
            {
                if (!reserved.contains(String.valueOf(key)))
                {
                    addRow(String.valueOf(key), String.valueOf(kvs.get(key)), Integer.parseInt(String.valueOf(types.get(key))));
                }
            }
        }

        LinearLayout ll = this.findViewById(R.id.mainLayout);
        for (int i = 0; i < ll.getChildCount(); i++)
        {
            if (ll.getChildAt(i) instanceof TextInputLayout)
            {
                TextInputLayout ti = (TextInputLayout) ll.getChildAt(i);
                for (int j = 0; j < ti.getChildCount(); j++)
                {
                    if (ti.getChildAt(j) instanceof FrameLayout)
                    {
                        FrameLayout fl = (FrameLayout) ti.getChildAt(j);
                        for (int k = 0; k < fl.getChildCount(); k++)
                        {
                            if (fl.getChildAt(k) instanceof TextInputEditText)
                            {
                                TextInputEditText et = (TextInputEditText) fl.getChildAt(k);

                                if (null == et)
                                {
                                    Log.d("LOADER", "et is null");
                                    continue;
                                }

                                String hint = ti.getHint().toString();

                                if (pref.contains(hint))
                                {
                                    try
                                    {
                                        et.setText(URLDecoder.decode(pref.getString(hint, ""), "UTF-8"));
                                    } catch (UnsupportedEncodingException e)
                                    {
                                        //Ignore
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}
