package net.jakewilliamson.sentry;

import java.util.LinkedHashMap;

class LRUCache<K, V> extends LinkedHashMap<K, V>
{
    private final int maxSize;
    private final CleanupCallback callback;

    public LRUCache(int maxSize, CleanupCallback callback)
    {
        super(maxSize + 1, 1, true);

        this.maxSize = maxSize;
        this.callback = callback;
    }

    @Override
    protected boolean removeEldestEntry(Entry<K, V> eldest)
    {
        if (size() > maxSize)
        {
            callback.cleanup(eldest);
            return true;
        }
        return false;
    }

    public interface CleanupCallback<K, V>
    {
        void cleanup(Entry<K, V> eldest);
    }
}
