package net.jakewilliamson.sentry;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.ContextWrapper;
import android.content.Intent;
import android.graphics.drawable.Icon;

class NotificationUtils extends ContextWrapper
{
    private static final String CHANNEL_ID = "net.jakewilliamson.sentry.ANDROID";
    private static final String CHANNEL_NAME = "ANDROID CHANNEL";
    private NotificationManager manager;

    public NotificationUtils(Context base)
    {
        super(base);
        createChannels();
    }

    private void createChannels()
    {
        NotificationChannel channel = new NotificationChannel(CHANNEL_ID, CHANNEL_NAME, NotificationManager.IMPORTANCE_HIGH);
        channel.enableVibration(true);
        getManager().createNotificationChannel(channel);
    }

    public NotificationManager getManager()
    {
        if (null == manager)
        {
            manager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
        }
        return manager;
    }

    public Notification.Builder showLockdown()
    {
        Intent intent = new Intent(this, SettingsFragment.class);
        PendingIntent pendingIntent = PendingIntent.getActivity(this, 666, intent, PendingIntent.FLAG_UPDATE_CURRENT);

        return new Notification.Builder(getApplicationContext(), CHANNEL_ID)
                .setContentTitle("Lockdown Mode Enabled")
                .setContentText("Tap to go to settings")
                .setSmallIcon(R.drawable.ic_warning_white_24dp)
                .setOngoing(true)
                .setContentIntent(pendingIntent)
                .setAutoCancel(false);
    }

    public Notification.Builder getNotification(String title, String body, boolean showActions)
    {
        if (showActions)
        {
            Intent blockIntent = new Intent(this, BlockUploadReceiver.class);
            PendingIntent pendingIntentBlock = PendingIntent.getBroadcast(this, 0, blockIntent, PendingIntent.FLAG_UPDATE_CURRENT);

            Intent allowIntent = new Intent(this, AllowUploadReceiver.class);
            PendingIntent pendingIntentAllow = PendingIntent.getBroadcast(this, 0, allowIntent, PendingIntent.FLAG_UPDATE_CURRENT);

            Notification.Action blockAction = new Notification.Action.Builder(
                    Icon.createWithResource(this, R.drawable.ic_block_black_24dp),
                    "Block",
                    pendingIntentBlock).build();

            Notification.Action allowAction = new Notification.Action.Builder(
                    Icon.createWithResource(this, R.drawable.ic_done_black_24dp),
                    "Allow",
                    pendingIntentAllow).build();


            return new Notification.Builder(getApplicationContext(), CHANNEL_ID)
                    .setContentTitle(title)
                    .setContentText(body)
                    .setSmallIcon(R.drawable.ic_warning_white_24dp)
                    .addAction(blockAction)
                    .addAction(allowAction)
                    .setAutoCancel(true);
        } else
        {
            return new Notification.Builder(getApplicationContext(), CHANNEL_ID)
                    .setContentTitle(title)
                    .setContentText(body)
                    .setSmallIcon(R.drawable.ic_warning_white_24dp)
                    .setAutoCancel(true);
        }
    }
}
