package net.jakewilliamson.sentry;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.net.VpnService;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.support.v4.content.LocalBroadcastManager;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.Toast;

import static net.jakewilliamson.sentry.Util.getDataState;
import static net.jakewilliamson.sentry.Util.getWifiState;


public class Sentry extends AppCompatActivity
{
    private static final int VPN_REQUEST_CODE = 0x0F;

    private boolean waitingForVPNStart;

    private boolean allowWifi = true;
    private boolean allowData = true;

    private String connectFailReason = "";

    private final BroadcastReceiver vpnStateReceiver = new BroadcastReceiver()
    {
        @Override
        public void onReceive(Context context, Intent intent)
        {
            if (LocalVPNService.BROADCAST_VPN_STATE.equals(intent.getAction()))
            {
                if (intent.getBooleanExtra("running", false))
                    waitingForVPNStart = false;
            }
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_local_vpn);

        long startTime = getIntent().getLongExtra("loadTime", 0);
        long currentTime = System.nanoTime();

        double loadTime = (currentTime - startTime) / 1000000000.0;

        Log.i("LoadingTime", String.valueOf(loadTime) + "s");

        final Button vpnButton = findViewById(R.id.vpn);
        final Button dataButton = findViewById(R.id.editData);
        final Button historyButton = findViewById(R.id.showHistory);
        vpnButton.setOnClickListener(new View.OnClickListener()
        {
            @Override
            public void onClick(View v)
            {
                if (LocalVPNService.isRunning())
                {
                    stopVPN();
                } else
                {
                    startVPN();
                }
            }
        });

        dataButton.setOnClickListener(new View.OnClickListener()
        {
            @Override
            public void onClick(View v)
            {
                editPersonalData();
            }
        });

        historyButton.setOnClickListener(new View.OnClickListener()
        {
            @Override
            public void onClick(View v)
            {
                showHistory();
            }
        });

        waitingForVPNStart = false;
        LocalBroadcastManager.getInstance(this).registerReceiver(vpnStateReceiver,
                new IntentFilter(LocalVPNService.BROADCAST_VPN_STATE));
    }

    private void showHistory()
    {
        startActivity(new Intent(this, History.class));
    }

    private void editPersonalData()
    {
        startActivity(new Intent(this, EditData.class));
    }

    private void startVPN()
    {
        if (!checkConnection())
        {
            Toast.makeText(getApplicationContext(), "VPN is not allowed while connected to " + connectFailReason,
                    Toast.LENGTH_LONG).show();
            return;
        }

        Intent vpnIntent = VpnService.prepare(this);
        if (vpnIntent != null)
            startActivityForResult(vpnIntent, VPN_REQUEST_CODE);
        else
            onActivityResult(VPN_REQUEST_CODE, RESULT_OK, null);
    }

    private void stopVPN()
    {
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data)
    {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == VPN_REQUEST_CODE && resultCode == RESULT_OK)
        {
            waitingForVPNStart = true;
            Toast.makeText(getApplicationContext(), "VPN is running, check notification bar",
                    Toast.LENGTH_LONG).show();
            startService(new Intent(this, LocalVPNService.class));
            enableButton(false);
        }
    }

    @Override
    protected void onResume()
    {
        super.onResume();

        enableButton(!waitingForVPNStart && !LocalVPNService.isRunning());
    }

    private void enableButton(boolean enable)
    {
        final Button vpnButton = findViewById(R.id.vpn);
        //vpnButton.setEnabled(enable);
        if (enable)
        {
            vpnButton.setText(R.string.start_vpn);
        } else
        {
            vpnButton.setText(R.string.stop_vpn);
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        getMenuInflater().inflate(R.menu.menu_local_vpn, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        switch (item.getItemId())
        {
            case R.id.action_settings:
                Intent i = new Intent(this, SettingsFragment.class);
                startActivity(i);
                return true;
            case R.id.action_about:
                Intent about = new Intent(this, About.class);
                startActivity(about);
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    private boolean checkConnection()
    {
        getSettings();
        getWifiState(this);
        getDataState(this);
        boolean onWifi = Util.connectedToWifi;
        boolean onData = Util.connectedToData;

        if (onWifi && allowWifi)
        {
            return true;
        } else if (onData && allowData)
        {
            return true;
        } else
        {
            if (onWifi)
            {
                connectFailReason = "WiFi";
            } else if (onData)
            {
                connectFailReason = "Network";
            }
            return false;
        }
    }

    private void getSettings()
    {
        SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(this);
        allowWifi = pref.getBoolean("wifiAllow", true);
        allowData = pref.getBoolean("dataAllow", true);
    }
}
