package net.jakewilliamson.sentry;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;

import java.io.Closeable;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

class Util
{
    private static final ArrayList<PackageInfoStruct> installedApps = new ArrayList<>();
    static boolean connectedToWifi = false;
    static boolean connectedToData = false;

    static String getSelfVersionName(Context context)
    {
        try
        {
            PackageInfo pInfo = context.getPackageManager().getPackageInfo(context.getPackageName(), 0);
            return pInfo.versionName;
        } catch (PackageManager.NameNotFoundException ex)
        {
            return ex.toString();
        }
    }

    static int getSelfVersionCode(Context context)
    {
        try
        {
            PackageInfo pInfo = context.getPackageManager().getPackageInfo(context.getPackageName(), 0);
            return pInfo.versionCode;
        } catch (PackageManager.NameNotFoundException ex)
        {
            return -1;
        }
    }

    public static void getInstalledApps(boolean getSystemApps, Context context)
    {
        List<PackageInfo> packs = context.getPackageManager().getInstalledPackages(0);
        ArrayList<PackageInfoStruct> ret = new ArrayList<>();
        for (int i = 0; i < packs.size(); i++)
        {
            PackageInfo p = packs.get(i);
            if ((!getSystemApps) && (p.versionName == null))
                continue;
            PackageInfoStruct newInfo = new PackageInfoStruct();
            newInfo.appName = p.applicationInfo.loadLabel(context.getPackageManager()).toString();
            newInfo.packageName = p.packageName;
            newInfo.versionName = p.versionName;
            newInfo.versionCode = p.versionCode;
            //newInfo.icon = p.applicationInfo.loadIcon(context.getPackageManager());

            installedApps.add(newInfo);
        }
    }

    public static void getWifiState(Context context)
    {
        ConnectivityManager manager = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo activeNetwork;
        if (manager != null)
        {
            activeNetwork = manager.getActiveNetworkInfo();
            connectedToWifi = activeNetwork.getType() == ConnectivityManager.TYPE_WIFI;
        }
    }

    public static void getDataState(Context context)
    {
        ConnectivityManager manager = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo activeNetwork;
        if (manager != null)
        {
            activeNetwork = manager.getActiveNetworkInfo();
            connectedToData = activeNetwork.getType() == ConnectivityManager.TYPE_MOBILE;
        }
    }

    public static void closeResources(Closeable... resources)
    {
        for (Closeable resource : resources)
        {
            try
            {
                resource.close();
            } catch (IOException e)
            {
                // Ignore
            }
        }
    }
}
